import { Injectable } from "@nestjs/common";
import { Cron, CronExpression } from "@nestjs/schedule";
import { InjectRepository } from "@nestjs/typeorm";
import { fetchAllProductCategory, fetchAllRetailers } from "src/helper/dutchie-plus-query";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import { ProductCategoryRepository } from "../product-category/product-category.repository";
import { ProductServiceRepository } from "../product-service/product-service.repository";
import { urlSlug } from "../../helper/common-helper";

@Injectable()
export class SchedulerService {
    constructor(
      @InjectRepository(ProductCategoryRepository)
      private readonly productCatRepo: ProductCategoryRepository,
      @InjectRepository(ProductServiceRepository)
      private readonly productServiceRepo: ProductServiceRepository
    ) {
    }

    // create or update dutchie plus category all retailers
    @Cron(CronExpression.EVERY_12_HOURS)
    async createUpdateDutchiePlusCategory() {
        console.log(">>> Run Create or Update Dutchie Plus Category Scheduler <<<", new Date());

        try {
            const graphQLClient = getGraphQLClient();
            const retailersQuery = fetchAllRetailers();
            const retailersData = await graphQLClient.request(retailersQuery);
            const retailers = retailersData.retailers;

            if (retailers.length > 0) {
                const categoryPromises = retailers.map(async (retailer) => {
                    const variables = {
                        retailerId: retailer.id
                    };
                    const categoryQuery = fetchAllProductCategory();
                    const categoryData = await graphQLClient.request(categoryQuery, variables);
                    return categoryData.menu.products.map((product) => product.category);
                });

                const retailerCategories = await Promise.all(categoryPromises);
                const allCategories = [...new Set(retailerCategories.flat())];

                const productCat = await this.productCatRepo.findAll();

                if (Object.keys(productCat).length > 0) {
                    const difference = allCategories.filter((category) =>
                      !productCat.some((c) => c.categoryName === category)
                    );
                    console.log("Product Categories Difference = ", difference);
                    // if difference found then create entities with difference product category
                    const createProductCategoryDtos = difference.map((item) => {
                        return {
                            categoryName: item,
                            urlSlug: urlSlug(item)
                        };
                    });

                    if (createProductCategoryDtos.length > 0) {
                        await this.productCatRepo.createEntities(createProductCategoryDtos);
                    }
                } else {
                    // if product categories table is empty in our local database then create all the category in local
                    const createProductCategoryDtos = allCategories.map((item) => {
                        return {
                            categoryName: item,
                            urlSlug: urlSlug(item)
                        };
                    });

                    if (createProductCategoryDtos.length > 0) {
                        await this.productCatRepo.createEntities(createProductCategoryDtos);
                    }
                }
            } else {
                console.log("Retailers Not Found!");
            }
            console.log(">>> End Create or Update Dutchie Plus Category Scheduler <<<", new Date());
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    // @Cron(CronExpression.EVERY_HOUR)
    // async createUpdateDutchiePlusProducts() {
    //     try {
    //         console.log(">>>Run Create or Update Dutchie Plus Products Scheduler<<<");
    //         const graphQLClient = getGraphQLClient();
    //         const query = fetchAllRetailers();
    //         const data = await graphQLClient.request(query);
    //         const retailers = data.retailers;
    //         if (retailers.length > 0) {
    //             for (let i = 0; i < retailers.length; i++) {
    //                 // find all products count based on retailers
    //                 const variables = {
    //                     retailerId: retailers[i].id
    //                 };
    //                 const fetchProductsCountQuery = fetchProductsCount();
    //                 const productCountData = await graphQLClient.request(
    //                   fetchProductsCountQuery,
    //                   variables
    //                 );
    //                 const productCount = productCountData["menu"].productsCount;
    //                 if (productCount > 0) {
    //                     // fetch all products from dutchie plus
    //                     const productsVariable = {
    //                         retailerId: retailers[i].id,
    //                         pagination: {
    //                             offset: 0,
    //                             limit: productCount
    //                         }
    //                     };
    //                     const fetchProductsQuery = fetchAllProductsForScheduler();
    //                     const productData = await graphQLClient.request(
    //                       fetchProductsQuery,
    //                       productsVariable
    //                     );
    //                     // fetch local database data
    //                     const productService = await this.productServiceRepo.findAll();
    //                     if (Object.keys(productService).length > 0) {
    //                         // get difference between local products and dutchie products
    //                         const differenceArray = productData["menu"]["products"].filter(
    //                           (obj: any) => {
    //                               return (
    //                                 productService.findIndex((o) => o.productId === obj.id) ===
    //                                 -1
    //                               );
    //                           }
    //                         );
    //                         console.log("Product Difference=", differenceArray);
    //                         for (let j = 0; j < differenceArray.length; j++) {
    //                             const brand = differenceArray[j].brand !== null ? differenceArray[j].brand.name : "";
    //                             const productServiceDto = {
    //                                 retailerId: retailers[i].id,
    //                                 retailerName: retailers[i].name,
    //                                 productId: differenceArray[j].id,
    //                                 productName: differenceArray[j].name,
    //                                 productSlug: differenceArray[j].slug,
    //                                 image: differenceArray[j].image,
    //                                 brandName: brand,
    //                                 category: differenceArray[j].category,
    //                                 variants: differenceArray[j].variants.length
    //                             };
    //                             await this.productServiceRepo.createEntity(
    //                               productServiceDto as CreateProductServiceDto
    //                             );
    //                         }
    //                     } else {
    //                         // if no product in database create an entity
    //                         for (let k = 0; k < productData['menu']['products'].length; k++) {
    //                             const brand = productData["menu"]["products"][k].brand !== null ? productData["menu"]["products"][k].brand.name : "";
    //                             const productServiceDto = {
    //                                 retailerId: retailers[i].id,
    //                                 retailerName: retailers[i].name,
    //                                 productId: productData["menu"]["products"][k].id,
    //                                 productName: productData["menu"]["products"][k].name,
    //                                 productSlug: productData["menu"]["products"][k].slug,
    //                                 image: productData["menu"]["products"][k].image,
    //                                 brandName: brand,
    //                                 category: productData["menu"]["products"][k].category,
    //                                 variants: productData["menu"]["products"][k].variants.length
    //                             };
    //                             await this.productServiceRepo.createEntity(
    //                               productServiceDto as CreateProductServiceDto
    //                             );
    //                         }
    //                     }
    //                 }
    //             }
    //         } else {
    //             console.log("Retailers Not Found!");
    //         }
    //     } catch (e) {
    //         console.log(e);
    //         return e;
    //     }
    // }
}
